from .hello_msg import *
from .utils import *
from .flags import *
from .cursor_lib import *
from .fps_clock import NFps
from .child_collector import NChildCollector
from .main_frame import NMainFrame
from .label import NLabel
from .image import NImage
from .timer import NTimer
from .win_button import NWinButton
from .simple_color_fade import NSimpleColorFade
from .win_animated_button import NWinAnimatedButton
from .checkbox import NCheckBox
from .checkbox_animated import NAnimatedCheckBox
from .simple_line_edit import NSimpleLineEdit
from .horizontal_slider import NHorizontalSlider
from .vertical_slider import NVerticalSlider


__version__ = '0.0.7'



import pygame
from . import *


cursors = {

}
system_cursors = {

}


def compile_cursors() -> None:
    for _cursor in dir(pygame):
        if not _cursor.startswith('SYSTEM_CURSOR'):
            continue
        system_cursors[_cursor[14:]] = getattr(pygame, _cursor)

    cursors['DEFAULT'] = pygame.mouse.get_cursor()
    cursors['EMPTY'] = pygame.cursors.Cursor((8, 8), (0, 0), *pygame.cursors.compile((
        '        ',
        '        ',
        '        ',
        '        ',
        '        ',
        '        ',
        '        ',
        '        ',
    )))



import os
import sys
import pygame
import random
import ctypes
from PIL import Image
try:
    import android  # type: ignore
    from android.permissions import request_permissions  # type: ignore
    from android.permissions import Permission  # type: ignore
    is_android = True
except ImportError:
    is_android = False
try:
    import jnius  # type: ignore
    is_jni = True
except Exception as _err:
    del _err
    is_jni = False
try:
    import win32api  # type: ignore
    import win32gui  # type: ignore
    import win32con  # type: ignore
    import win32print  # type: ignore
    is_winapi = True
except ImportError:
    is_winapi = False


encoding = sys.getdefaultencoding()
is_windows = not is_android and hasattr(ctypes, 'windll')
cur_path = os.getcwd()


def request_android_default_permissions() -> None:
    if not is_android:
        return
    request_permissions([
        Permission.READ_EXTERNAL_STORAGE,
        Permission.WRITE_EXTERNAL_STORAGE,
        Permission.INTERNET
    ])


def round_tuple(_tuple: any) -> tuple:
    return tuple(round(_x) for _x in _tuple)


def random_float(a: float, b: float) -> float:
    return random.random() * (b - a) + a


def get_dpi() -> int:
    if is_android and is_jni:
        return jnius.autoclass('android.util.DisplayMetrics')().getDeviceDensity()
    if is_windows and is_winapi:
        hdc = win32gui.GetDC(0)
        result = max(
            win32print.GetDeviceCaps(hdc, win32con.LOGPIXELSX), win32print.GetDeviceCaps(hdc, win32con.LOGPIXELSY)
        )
        return result
    return 0


def p(*path) -> str:
    return os.path.join(cur_path, *path)


def image_to_surface(image: Image.Image) -> pygame.Surface:
    return pygame.image.frombuffer(
        image.tobytes(),
        image.size,
        image.mode  # type: ignore
    )


def surface_to_image(surface: pygame.Surface, is_flipped: bool = False) -> Image.Image:
    img_mode = 'RGB' if surface.get_alpha() is None else 'RGBA'
    return Image.frombytes(
        img_mode,  # type: ignore
        surface.get_size(),
        pygame.image.tostring(
            surface,
            img_mode,  # type: ignore
            is_flipped
        )
    )


def is_colliding(child: any, xy: tuple, offset_x: int = 0, offset_y: int = 0) -> bool:
    return child.w + child.x + offset_x > xy[0] >= child.x + offset_x and\
           child.h + child.y + offset_y > xy[1] >= child.y + offset_y


def is_colliding_rect(rect: any, xy: tuple, offset_x: int = 0, offset_y: int = 0) -> bool:
    return rect[2] + rect[0] + offset_x > xy[0] >= rect[0] + offset_x and\
           rect[3] + rect[1] + offset_y > xy[1] >= rect[1] + offset_y


def random_color(use_alpha: bool = False) -> tuple:
    return tuple(
        random.randint(0, 255) for _x in range(4 if use_alpha else 3)
    )



import pygame
from . import *


class NSimpleLineEdit:
    def __init__(
            self,
            parent: any,
            font: pygame.font.Font,
            xy: tuple,
            text: str = '',
            color: tuple = (0, 0, 0),
            blink_time: float = 0.75,
            bg_color: tuple = (0, 0, 0),
            show_bg: bool = False,
            auto_size: bool = True,
            anti_alias: bool = True,
            stretch: bool = False
    ) -> None:
        super(NSimpleLineEdit, self).__init__()
        self.font = font
        self.x, self.y = xy
        self.w, self.h = 0, 0
        self._width, self._height = self.w, self.h
        self.anti_alias = anti_alias
        self.auto_size = auto_size
        self.show_bg = show_bg
        self.stretch = stretch
        self.hook_mouse = True
        self.is_visible = True
        self.is_enabled = True
        self.is_focusable = True
        self.enable_scroll = True
        self.usable = True
        self.blink_symbol = '|'
        self.no_blink_symbol = ' '
        self.blink = self.no_blink_symbol
        self.blink_time = blink_time
        self.text = text
        self.x_offset = 0
        self.y_offset = 0
        self.blink_pos = 0
        self.bg_border_radius = 0
        self.multi_lines_align = LABEL_ALIGN_LEFT
        self.surface: pygame.Surface = None  # type: ignore
        self.timer = NTimer(None, self.blink_time, auto_start=False).set('on_tick', self.on_blink_tick)
        self.cursor = cursors.get('DEFAULT')
        self.color = color
        self.bg_color = bg_color
        self.z_order = 0
        self.tag = ''
        self.id = ''
        self.set('text', text)
        if parent:
            parent.add_child(self)

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        if name == 'text':
            self.blink_pos = len(value)
        if name in ('text', 'color', 'anti_alias', 'mutli_line_align', 'stretch', 'auto_size'):
            self.redraw(self.text)
        return self

    def redraw(self, text: str) -> None:
        if text.count('\n') > 0:
            text = text.replace('\n', '')
        self.surface = self.font.render(
            text[:self.blink_pos] + self.blink + text[self.blink_pos:],
            self.anti_alias,
            self.color
        )
        self._width, self._height = self.surface.get_size()
        if self.auto_size:
            self.w, self.h = self._width, self._height
        elif self.stretch:
            self.surface = pygame.transform.scale(self.surface, (self.w, self.h))
        return

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_visible:
            return
        self.timer.draw(surface, delta, scroll_x, scroll_y)
        if not self.enable_scroll:
            scroll_x = scroll_y = 0
        if self.show_bg:
            pygame.draw.rect(
                surface,
                self.bg_color,
                (self.x + scroll_x, self.y + scroll_y, self.w, self.h),
                0,
                self.bg_border_radius
            )
        surface.blit(
            self.surface,
            (self.x + self.x_offset + scroll_x, self.y + self.y_offset + scroll_y),
            None if self.auto_size else (0, 0, self.w - 1, self.h)
        )

    def on_blink_tick(self, delta: float) -> None:
        self.blink = self.no_blink_symbol if self.blink == self.blink_symbol else self.blink_symbol
        self.redraw(self.text)

    def process_key(self, event: pygame.event.Event, text: str) -> None:
        if event.key == pygame.K_BACKSPACE:
            self.timer.current_rate = 0.0
            self.blink = self.blink_symbol
            if len(text) > 0:
                self.text = self.text[:self.blink_pos - 1] + self.text[self.blink_pos:]
                self.blink_pos -= 1
        elif event.key == pygame.K_RIGHT:
            if len(self.text) > self.blink_pos:
                self.blink_pos += 1
        elif event.key == pygame.K_LEFT:
            if self.blink_pos > 0:
                self.blink_pos -= 1
        elif event.key == pygame.K_DELETE:
            self.text = self.text[:self.blink_pos] + self.text[self.blink_pos + 1:]
        elif event.unicode:
            self.timer.current_rate = 0.0
            self.blink = self.no_blink_symbol
            self.text = self.text[:self.blink_pos] + event.unicode + self.text[self.blink_pos:]
            self.blink_pos += len(event.unicode)

    def _on_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_leave(event.pos)  # type: ignore

    def _on_mouse_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_enter(event.pos)  # type: ignore

    def _on_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_move(  # type: ignore
                event.pos, event.rel, event.buttons, hasattr(event, 'touch') and event.touch
            )

    def _on_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_down(event.pos, event.button)  # type: ignore

    def _on_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_up(event.pos, event.button)  # type: ignore
        if event.button == pygame.BUTTON_LEFT and hasattr(self, 'on_click'):
            self.on_click(event.pos)

    def _on_focus_enter(self, event: pygame.event.Event, bind: bool) -> None:
        self.blink = self.blink_symbol
        self.redraw(self.text)
        self.timer.run()
        pygame.key.start_text_input()
        if bind:
            self.on_focus_enter(event.pos, event.button)  # type: ignore

    def _on_focus_leave(self, event: pygame.event.Event, bind: bool) -> None:
        pygame.key.stop_text_input()
        self.timer.stop()
        self.timer.current_rate = 0.0
        self.blink = self.no_blink_symbol
        self.redraw(self.text)
        if bind:
            self.on_focus_leave(event.pos, event.button)  # type: ignore

    def _on_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        self.process_key(event, self.text)
        self.redraw(self.text)
        if bind:
            self.on_key_down(event)  # type: ignore

    def _on_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_up(event)  # type: ignore



import pygame
from . import *


class NMainFrame:
    def __init__(
            self,
            surface: pygame.Surface
    ) -> None:
        super(NMainFrame, self).__init__()
        self.surface = surface
        self.w, self.h = self.surface.get_size()
        self.processes = {
            pygame.QUIT: 'on_quit',
            pygame.WINDOWRESIZED: 'on_resize',
            pygame.KEYDOWN: 'on_global_key_down',
            pygame.KEYUP: 'on_global_key_up',
            pygame.MOUSEBUTTONDOWN: 'on_global_mouse_down',
            pygame.MOUSEBUTTONUP: 'on_global_mouse_up',
            pygame.MOUSEMOTION: 'on_global_mouse_move',
            pygame.MOUSEWHEEL: 'on_global_mouse_wheel'
        }
        self.hook_mouse = True
        self.is_focusable = True
        self.is_mouse_left_down = False
        self.scroll_x, self.scroll_y = 0, 0
        if not cursors:
            compile_cursors()
        self.cursor = cursors.get('DEFAULT')
        self.current_cursor = self.cursor
        self.last_hover = self
        self.last_focus = self
        self.child = NChildCollector()
        self.find_by_tag = self.child.find_by_tag
        self.find_by_id = self.child.find_by_id
        self.add_child = self.child.add_child
        self.remove_child = self.child.remove_child
        self.export_child = self.child.export_child
        self.import_child = self.child.import_child
        self.sort_child = self.child.sort

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        return self

    def process_events(self, events: list) -> list:
        for event in events:
            func = self.processes.get(event.type)
            if not func:
                continue
            getattr(self, '_' + func)(event, hasattr(self, func))
        return events

    def draw(
            self,
            delta: float
    ) -> None:
        for child in self.child.child:
            child.draw(self.surface, delta, self.scroll_x, self.scroll_y)

    def _on_quit(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_quit()  # type: ignore

    def _on_resize(self, event: pygame.event.Event, bind: bool) -> None:
        self.w, self.h = event.x, event.y
        if bind:
            self.on_resize(event.x, event.y)  # type: ignore

    def _on_global_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        if hasattr(self.last_focus, '_on_key_down'):
            self.last_focus._on_key_down(event, hasattr(self.last_focus, 'on_key_down'))
        if bind:
            self.on_global_key_down(event)  # type: ignore

    def _on_global_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if hasattr(self.last_focus, '_on_key_up'):
            self.last_focus._on_key_up(event, hasattr(self.last_focus, 'on_key_up'))
        if bind:
            self.on_global_key_up(event)  # type: ignore

    def _on_global_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if event.button == pygame.BUTTON_LEFT:
            self.is_mouse_left_down = True
            hover = self
            for child in self.child.child[::-1]:
                if child.usable and child.hook_mouse and child.is_enabled\
                        and is_colliding(child, event.pos, self.scroll_x, self.scroll_y):
                    hover = child
                    break
            if not hover == self.last_hover:
                if hasattr(self.last_hover, '_on_mouse_leave'):
                    self.last_hover._on_mouse_leave(event, hasattr(self.last_hover, 'on_mouse_leave'))
                self.last_hover = hover
                if not self.last_hover.cursor == self.current_cursor:
                    pygame.mouse.set_cursor(self.last_hover.cursor)
                    self.current_cursor = self.last_hover.cursor
                if hasattr(self.last_hover, '_on_mouse_enter'):
                    self.last_hover._on_mouse_enter(event, hasattr(self.last_hover, 'on_mouse_enter'))
            focus = self
            for child in self.child.child[::-1]:
                if child.usable and child.hook_mouse and child.is_enabled and \
                        is_colliding(child, event.pos, self.scroll_x, self.scroll_y):
                    focus = child
                    break
            if not focus == self.last_focus:
                if hasattr(self.last_focus, '_on_focus_leave'):
                    self.last_focus._on_focus_leave(event, hasattr(self.last_focus, 'on_focus_leave'))
                self.last_focus = focus
                if hasattr(self.last_focus, '_on_focus_enter'):
                    self.last_focus._on_focus_enter(event, hasattr(self.last_focus, 'on_focus_enter'))
        if hasattr(self.last_hover, '_on_mouse_down'):
            self.last_hover._on_mouse_down(event, hasattr(self.last_hover, 'on_mouse_down'))
        if bind:
            self.on_global_mouse_down(event.pos, event.button)  # type: ignore

    def _on_global_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if event.button == pygame.BUTTON_LEFT:
            self.is_mouse_left_down = False
        if hasattr(self.last_hover, '_on_mouse_up'):
            self.last_hover._on_mouse_up(event, hasattr(self.last_hover, 'on_mouse_up'))
        if event.button == pygame.BUTTON_LEFT:
            hover = self
            for child in self.child.child[::-1]:
                if child.usable and child.hook_mouse and child.is_enabled and\
                        is_colliding(child, event.pos, self.scroll_x, self.scroll_y):
                    hover = child
                    break
            if not hover == self.last_hover:
                if hasattr(self.last_hover, '_on_mouse_leave'):
                    self.last_hover._on_mouse_leave(event, hasattr(self.last_hover, 'on_mouse_leave'))
                self.last_hover = hover
                if not self.last_hover.cursor == self.current_cursor:
                    pygame.mouse.set_cursor(self.last_hover.cursor)
                    self.current_cursor = self.last_hover.cursor
                if hasattr(self.last_hover, '_on_mouse_enter'):
                    self.last_hover._on_mouse_enter(event, hasattr(self.last_hover, 'on_mouse_enter'))
            elif is_android:  # TODO: do something
                if hasattr(self.last_hover, '_on_mouse_leave'):
                    self.last_hover._on_mouse_leave(event, hasattr(self.last_hover, 'on_mouse_leave'))
                self.last_hover = self
                if hasattr(self.last_hover, '_on_mouse_enter'):
                    self.last_hover._on_mouse_enter(event, hasattr(self.last_hover, 'on_mouse_enter'))
        if bind:
            self.on_global_mouse_up(event.pos, event.button)  # type: ignore

    def _on_global_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if not self.is_mouse_left_down:
            hover = self
            for child in self.child.child[::-1]:
                if child.usable and child.hook_mouse and child.is_enabled and\
                        is_colliding(child, event.pos, self.scroll_x, self.scroll_y):
                    hover = child
                    break
            if not hover == self.last_hover:
                if hasattr(self.last_hover, '_on_mouse_leave'):
                    self.last_hover._on_mouse_leave(event, hasattr(self.last_hover, 'on_mouse_leave'))
                self.last_hover = hover
                if not self.last_hover.cursor == self.current_cursor:
                    pygame.mouse.set_cursor(self.last_hover.cursor)
                    self.current_cursor = self.last_hover.cursor
                if hasattr(self.last_hover, '_on_mouse_enter'):
                    self.last_hover._on_mouse_enter(event, hasattr(self.last_hover, 'on_mouse_enter'))
        if hasattr(self.last_hover, '_on_mouse_move'):
            self.last_hover._on_mouse_move(event, hasattr(self.last_hover, 'on_mouse_move'))
        if bind:
            self.on_global_mouse_move(event.pos)  # type: ignore

    def _on_global_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if hasattr(self.last_focus, '_on_mouse_wheel'):
            self.last_focus._on_mouse_wheel(event, hasattr(self.last_focus, 'on_mouse_wheel'))
        if bind:
            self.on_global_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_leave(event.pos)  # type: ignore

    def _on_mouse_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_enter(event.pos)  # type: ignore

    def _on_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_move(  # type: ignore
                event.pos, event.rel, event.buttons, hasattr(event, 'touch') and event.touch
            )

    def _on_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_down(event.pos, event.button)  # type: ignore

    def _on_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_up(event.pos, event.button)  # type: ignore
        if event.button == pygame.BUTTON_LEFT and hasattr(self, 'on_click'):
            self.on_click(event.pos)

    def _on_focus_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_enter(event.pos, event.button)  # type: ignore

    def _on_focus_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_leave(event.pos, event.button)  # type: ignore

    def _on_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_down(event)  # type: ignore

    def _on_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_up(event)  # type: ignore



import time


class NFps:
    def __init__(
            self,
            fps: any,
            unlocked: bool = False,
            smooth_fix: bool = False,
            time_function: any = time.time
    ) -> None:
        super(NFps, self).__init__()
        self.fps = float(fps)
        self.frame_rate = 1 / fps
        self.delta = self.frame_rate
        self.unlocked = unlocked
        self.smooth_fix = smooth_fix
        self.speed_hack = 1.0
        self.time_func = time_function
        self.last_tick = self.time_func()

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        if name == 'time_func':
            self.last_tick = self.time_func()
        return self

    def tick(self) -> bool:
        now = self.time_func()
        if not self.unlocked and now < self.last_tick + self.frame_rate:
            return False
        if self.smooth_fix:
            self.delta = self.frame_rate * self.speed_hack
            self.last_tick += self.frame_rate
        else:
            self.delta = (now - self.last_tick) * self.speed_hack
            self.last_tick = now
        return True

    def get_fps_float(self) -> float:
        try:
            return self.speed_hack / self.delta
        except ZeroDivisionError:
            return self.fps

    def get_fps_int(self) -> int:
        return round(self.get_fps_float())



from . import *


class NChildCollector:
    def __init__(
            self
    ) -> None:
        super(NChildCollector, self).__init__()
        self.child = []

    def add_child(self, child: any) -> None:
        if not hasattr(child, 'z_order'):
            child.z_order = 0
        return self.add_child_sorted(child, child.z_order)

    def add_child_sorted(self, child: any, z_order: int) -> None:
        self.child.append(child)
        if len(self.child) <= 0:
            return
        self.sort()

    def remove_child(self, child: any) -> None:
        self.child.remove(child)

    def has_child(self, child: any) -> bool:
        return self.child.count(child) >= 1

    def find_by_tag(self, tag: str) -> tuple:
        return tuple(_x for _x in self.child[::-1] if _x.tag == tag)

    def find_by_id(self, _id: str) -> any:
        for child in self.child[::-1]:
            if child.id == _id:
                return child
        return None

    def sort(self) -> None:
        self.child.sort(key=lambda x: x.z_order, reverse=False)

    def get_child(self) -> list:
        return self.child

    def export_child(self) -> tuple:
        return tuple(self.child)

    def import_child(self, child: tuple) -> None:
        self.child = list(child)

    def clear(self) -> None:
        self.child.clear()



import pygame
from . import *


class NVerticalSlider:
    def __init__(
            self,
            parent: any,
            xy: tuple,
            size: tuple = (22, 160),
            value: any = 0,
            min_value: any = 0,
            max_value: any = 99
    ) -> None:
        super(NVerticalSlider, self).__init__()
        self.x, self.y = xy
        self.w, self.h = size
        self.hook_mouse = True
        self.is_visible = True
        self.is_enabled = True
        self.is_focusable = True
        self.enable_scroll = True
        self.usable = True
        self.value = value
        self.min_value = min_value
        self.max_value = max_value
        self.single_step = 1
        self.page_step = 10
        self.slider_height = 10
        self.bar_hovered = False
        self.bar_focused = False
        self.down_it = False
        self.bar_width = 4
        self.bar_top = 0
        self.lsx, self.lsy = 0, 0
        self.last_y_focus = 0
        self.bar_border_color = (214, 214, 214)
        self.bar_color = (231, 234, 234)
        self.color = (0, 120, 215)
        self.hover_color = (23, 23, 23)
        self.focus_color = (204, 204, 204)
        self.current_color = self.color
        self.cursor = cursors.get('DEFAULT')
        self.z_order = 0
        self.tag = ''
        self.id = ''
        self.recalc_pos()
        if parent:
            parent.add_child(self)

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        if name in ('bar_width', ):
            self.recalc_pos()
        return self

    def recalc_pos(self) -> None:
        self.bar_top = round(self.w / 2 - self.bar_width / 2)

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_visible:
            return
        if not self.enable_scroll:
            scroll_x = scroll_y = 0
        self.lsx, self.lsy = scroll_x, scroll_y
        pygame.draw.rect(
            surface,
            self.bar_border_color,
            (self.x + self.bar_top + scroll_x, self.y + scroll_y, self.bar_width, self.h),
            1
        )
        pygame.draw.rect(
            surface,
            self.bar_color,
            (self.x + self.bar_top + scroll_x + 1, self.y + scroll_y + 1, self.bar_width - 2, self.h - 2),
        )
        pygame.draw.rect(
            surface,
            self.current_color,
            (self.x + scroll_x, self.y + scroll_y + (self.h - self.slider_height - (self.value - self.min_value) /
                                                     (self.max_value - self.min_value) * (self.h - self.slider_height)),
             self.w, self.slider_height)
        )

    def _on_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if not self.bar_focused:
            self.bar_hovered = False
            self.current_color = self.color
        if bind:
            self.on_mouse_leave(event.pos)  # type: ignore

    def _on_mouse_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_enter(event.pos)  # type: ignore

    def _on_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if self.bar_focused:
            y = event.pos[1] - self.y - self.lsy - self.last_y_focus
            value = (self.h - self.slider_height - y) / (self.h - self.slider_height) *\
                    (self.max_value - self.min_value) + self.min_value
            if value < self.min_value:
                value = self.min_value
            elif value > self.max_value:
                value = self.max_value
            self.value = value
            if hasattr(self, 'on_change'):
                self.on_change(value)
        elif not self.down_it:
            if is_colliding_rect(
                    (self.x + self.lsx, self.y + self.lsy + (self.h - self.slider_height -
                                                             (self.value - self.min_value) /
                                                             (self.max_value - self.min_value) *
                                                             (self.h - self.slider_height)), self.w,
                     self.slider_height), event.pos
            ):
                if not self.bar_hovered:
                    self.bar_hovered = True
                    self.current_color = self.hover_color
            elif self.bar_hovered:
                self.bar_hovered = False
                self.current_color = self.color
        if bind:
            self.on_mouse_move(  # type: ignore
                event.pos, event.rel, event.buttons, hasattr(event, 'touch') and event.touch
            )

    def _on_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if event.button == pygame.BUTTON_LEFT:
            if self.bar_hovered:
                self.bar_focused = True
                self.current_color = self.focus_color
                self.last_y_focus = event.pos[1] - self.y - self.lsy - self.h + self.slider_height +\
                                    ((self.value - self.min_value) / (self.max_value - self.min_value) *
                                     (self.h - self.slider_height))
            else:
                self.down_it = True
                if event.pos[1] > (
                        self.y + self.lsy + self.h - self.slider_height - ((self.value - self.min_value) /
                                                                           (self.max_value - self.min_value) *
                                                                           (self.h - self.slider_height)) +
                        self.slider_height / 2
                ):
                    self.value -= self.page_step
                    if self.value < self.min_value:
                        self.value = self.min_value
                else:
                    self.value += self.page_step
                    if self.value > self.max_value:
                        self.value = self.max_value
                if hasattr(self, 'on_change'):
                    self.on_change(self.value)
        if bind:
            self.on_mouse_down(event.pos, event.button)  # type: ignore

    def _on_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_up(event.pos, event.button)  # type: ignore
        if event.button == pygame.BUTTON_LEFT:
            self.down_it = False
            if self.bar_focused:
                self.bar_focused = False
                self.current_color = self.hover_color
            if hasattr(self, 'on_click'):
                self.on_click(event.pos)

    def _on_focus_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_enter(event.pos, event.button)  # type: ignore

    def _on_focus_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_leave(event.pos, event.button)  # type: ignore

    def _on_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_down(event)  # type: ignore

    def _on_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_up(event)  # type: ignore



import time
import pygame
from . import *


class NTimer:
    def __init__(
            self,
            parent: any,
            rate: any,
            repeat: bool = True,
            smooth_fix: bool = True,
            auto_start: bool = True,
            first_tick: bool = False
    ) -> None:
        super(NTimer, self).__init__()
        self.rate = float(rate)
        self.smooth_fix = smooth_fix
        self.repeat = repeat
        self.first_tick = first_tick
        self.usable = False
        self.is_enabled = False
        self.z_order = 0
        self.id = 0
        self.current_rate = 0.0
        if auto_start:
            self.run()
        if parent:
            parent.add_child(self)

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        return self

    def on_tick(self, delta: float) -> None:
        pass

    def run(self) -> None:
        self.is_enabled = True
        self.current_rate = self.rate if self.first_tick else 0.0

    def stop(self) -> None:
        self.is_enabled = False

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_enabled:
            return
        self.current_rate += delta
        while self.current_rate >= self.rate:
            if self.smooth_fix:
                self.current_rate -= self.rate
                self.on_tick(self.rate)
            else:
                self.on_tick(self.current_rate)
                self.current_rate = 0
            if not self.repeat:
                self.is_enabled = False



import os
import sys


is_already_inited = sys.modules.get('pygame') is not None
hide_msg = os.getenv('PYGAME_HIDE_SUPPORT_PROMPT') is not None
if is_already_inited:
    if not hide_msg:
        print('pynex integrated')
elif not hide_msg:
    os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = 'True'
    import pygame  # type: ignore
    os.unsetenv('PYGAME_HIDE_SUPPORT_PROMPT')
    print('pygame {} (SDL {}.{}.{}, Python {}.{}.{}, PyNex)'.format(
        pygame.ver, *pygame.get_sdl_version() + sys.version_info[0:3]
    ))
    print('Hello from the pygame community. https://www.pygame.org/contribute.html')
__all__ = ()



import pygame
from . import *


class NSimpleColorFade:
    def __init__(
            self,
            parent: any,
            from_color: tuple = (0, 0, 0),
            to_color: tuple = (255, 255, 255),
            time: any = 1,
            auto_start: bool = True
    ) -> None:
        super(NSimpleColorFade, self).__init__()
        self.from_color = from_color
        self.to_color = to_color
        self.time = float(time)
        self.speed = (0.0, 0.0, 0.0)
        self.current_color = [
            float(self.to_color[0]),
            float(self.to_color[1]),
            float(self.to_color[2])
        ]
        self.color = round_tuple(self.current_color)
        self.current_rate = 0.0
        self.id = 0
        self.usable = False
        self.is_enabled = False
        if auto_start:
            self.create(from_color, to_color, time)
        if parent:
            parent.add_child(self)

    def create(self, from_color: tuple, to_color: tuple, time: any, should_enable: bool = True) -> None:
        self.from_color = from_color
        self.to_color = to_color
        self.time = float(time)
        self.speed = (
            (self.to_color[0] - self.from_color[0]) / self.time,
            (self.to_color[1] - self.from_color[1]) / self.time,
            (self.to_color[2] - self.from_color[2]) / self.time
        )
        self.current_color[0] = float(self.from_color[0])
        self.current_color[1] = float(self.from_color[1])
        self.current_color[2] = float(self.from_color[2])
        self.color = round_tuple(self.color)
        self.current_rate = 0.0
        if should_enable:
            self.is_enabled = True

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_enabled:
            return
        self.current_color[0] += self.speed[0] * delta
        self.current_color[1] += self.speed[1] * delta
        self.current_color[2] += self.speed[2] * delta
        self.current_rate += delta
        if self.current_rate >= self.time:
            self.is_enabled = False
            self.current_rate = 0.0
            self.current_color[0] = float(self.to_color[0])
            self.current_color[1] = float(self.to_color[1])
            self.current_color[2] = float(self.to_color[2])
        self.color = round_tuple(self.current_color)



import pygame
from . import *


class NLabel:
    def __init__(
            self,
            parent: any,
            font: pygame.font.Font,
            xy: tuple,
            text: str,
            color: tuple = (0, 0, 0),
            bg_color: tuple = (0, 0, 0),
            show_bg: bool = False,
            auto_size: bool = True,
            anti_alias: bool = True,
            stretch: bool = False
    ) -> None:
        super(NLabel, self).__init__()
        self.font = font
        self.x, self.y = xy
        self.w, self.h = 0, 0
        self._width, self._height = self.w, self.h
        self.anti_alias = anti_alias
        self.auto_size = auto_size
        self.show_bg = show_bg
        self.stretch = stretch
        self.hook_mouse = True
        self.is_visible = True
        self.is_enabled = True
        self.is_focusable = True
        self.enable_scroll = True
        self.usable = True
        self.text = text
        self.x_offset = 0
        self.y_offset = 0
        self.bg_border_radius = 0
        self.multi_lines_align = LABEL_ALIGN_LEFT
        self.surface: pygame.Surface = None  # type: ignore
        self.cursor = cursors.get('DEFAULT')
        self.color = color
        self.bg_color = bg_color
        self.z_order = 0
        self.tag = ''
        self.id = ''
        self.set('text', text)
        if parent:
            parent.add_child(self)

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        if name in ('text', 'color', 'anti_alias', 'mutli_line_align', 'stretch', 'auto_size'):
            self.redraw()
        return self

    def redraw(self) -> None:
        if self.text.count('\n') <= 0:
            self.surface = self.font.render(
                self.text,
                self.anti_alias,
                self.color
            )
            self._width, self._height = self.surface.get_size()
            if self.auto_size:
                self.w, self.h = self._width, self._height
            elif self.stretch:
                self.surface = pygame.transform.scale(self.surface, (self.w, self.h))
            return
        total_height = 0
        max_width = 0
        surfaces = []
        heights = []
        for _text in self.text.split('\n'):
            _surface = self.font.render(
                _text,
                self.anti_alias,
                self.color
            )
            total_height += _surface.get_height()
            max_width = max(max_width, _surface.get_width())
            surfaces.append(_surface)
            heights.append(_surface.get_height())
        self._width, self._height = max_width, total_height
        self.surface = pygame.Surface((max_width, total_height), pygame.SRCALPHA, 32)
        total_height = 0
        for _num, _surface in enumerate(surfaces):
            self.surface.blit(
                _surface, (
                    0 if self.multi_lines_align == LABEL_ALIGN_LEFT else (
                        max_width - _surface.get_width() if self.multi_lines_align == LABEL_ALIGN_RIGHT else
                        round(max_width / 2 - _surface.get_width() / 2)
                    ),
                    total_height
                )
            )
            total_height += heights[_num]
        if self.auto_size:
            self.w, self.h = self._width, self._height
        elif self.stretch:
            self.surface = pygame.transform.scale(self.surface, (self.w, self.h))

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_visible:
            return
        if not self.enable_scroll:
            scroll_x = scroll_y = 0
        if self.show_bg:
            pygame.draw.rect(
                surface,
                self.bg_color,
                (self.x + scroll_x, self.y + scroll_y, self.w, self.h),
                0,
                self.bg_border_radius
            )
        surface.blit(
            self.surface,
            (self.x + self.x_offset + scroll_x, self.y + self.y_offset + scroll_y),
            None if self.auto_size else (0, 0, self.w - 1, self.h)
        )

    def _on_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_leave(event.pos)  # type: ignore

    def _on_mouse_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_enter(event.pos)  # type: ignore

    def _on_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_move(  # type: ignore
                event.pos, event.rel, event.buttons, hasattr(event, 'touch') and event.touch
            )

    def _on_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_down(event.pos, event.button)  # type: ignore

    def _on_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_up(event.pos, event.button)  # type: ignore
        if event.button == pygame.BUTTON_LEFT and hasattr(self, 'on_click'):
            self.on_click(event.pos)

    def _on_focus_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_enter(event.pos, event.button)  # type: ignore

    def _on_focus_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_leave(event.pos, event.button)  # type: ignore

    def _on_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_down(event)  # type: ignore

    def _on_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_up(event)  # type: ignore



import pygame
from . import *

FORCE_FULL_SCREEN = pygame.FULLSCREEN if is_android else 0
FORCE_SCALED = pygame.SCALED if is_android else 0

LABEL_ALIGN_LEFT = 0
LABEL_ALIGN_CENTER = 1
LABEL_ALIGN_RIGHT = 2

WIN_BUTTON_ALIGN_LEFT = WIN_BUTTON_ALIGN_TOP = 0
WIN_BUTTON_ALIGN_CENTER = 1
WIN_BUTTON_ALIGN_RIGHT = WIN_BUTTON_ALIGN_BOTTOM = 2



import os
import sys
import shutil
from setuptools import setup, find_packages
from pynex import __version__ as version


readme = open(os.path.join(os.path.dirname(__file__), 'README.md'), 'r').read()
requirements = ['pygame', 'Pillow']
for i in sys.argv[1:]:
    if not i.lower() == 'sdist' and not i.lower() == 'bdist_wheel':
        continue
    try:
        shutil.rmtree(os.path.join(os.getcwd(), 'build'))
    except Exception as __err:
        if __err:
            'Just for PyCharm'
    try:
        shutil.rmtree(os.path.join(os.getcwd(), 'dist'))
    except Exception as __err:
        if __err:
            'Just for PyCharm'
    try:
        shutil.rmtree(os.path.join(os.getcwd(), 'pynex.egg-info'))
    except Exception as __err:
        if __err:
            'Just for PyCharm'
    break


setup(
    name="pynex",
    author="Pixelsuft",
    url="https://github.com/Pixelsuft/pynex",
    project_urls={
        "Readme": "https://github.com/Pixelsuft/pynex/blob/main/README.MD",
        "Example": "https://github.com/Pixelsuft/pynex/blob/main/main.py",
        "Issue tracker": "https://github.com/Pixelsuft/pynex/issues",
    },
    version=version,
    packages=find_packages(),
    license="MIT",
    description="PyGame-based GUI Library.",
    long_description=readme,
    long_description_content_type="text/markdown",
    include_package_data=True,
    install_requires=requirements,
    python_requires=">=3.6",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    zip_safe=False,
    py_modules=["pynex"],
    package_dir={'': '.'},
    keywords="pynex"
)



import pygame
from . import *


class NImage:
    def __init__(
            self,
            parent: any,
            image: pygame.Surface,
            xy: tuple,
            auto_size: bool = True,
            stretch: bool = False
    ) -> None:
        super(NImage, self).__init__()
        self.image = image
        self.x, self.y = xy
        self.w, self.h = 0, 0
        self._width, self._height = self.w, self.h
        self.hook_mouse = True
        self.stretch = stretch
        self.auto_size = auto_size
        self.is_visible = True
        self.is_enabled = True
        self.is_focusable = True
        self.enable_scroll = True
        self.usable = True
        self.surface = self.image
        self.cursor = cursors.get('DEFAULT')
        self.z_order = 0
        self.tag = ''
        self.id = ''
        self.redraw()
        if parent:
            parent.add_child(self)

    def set(self, name: str, value: any) -> any:
        setattr(self, name, value)
        if name in ('stretch', 'image', 'w', 'h'):
            self.redraw()
        return self

    def redraw(self) -> None:
        if self.stretch:
            self._width, self._height = self.w, self.h
            self.surface = pygame.transform.scale(self.image, (self.w, self.h))
        else:
            self.surface = self.image
            self._width, self._height = self.surface.get_size()
            if self.auto_size:
                self.w, self.h = self._width, self._height

    def draw(self, surface: pygame.Surface, delta: float, scroll_x: int, scroll_y: int) -> None:
        if not self.is_visible:
            return
        if not self.enable_scroll:
            scroll_x = scroll_y = 0
        surface.blit(
            self.surface,
            (self.x + scroll_x, self.y + scroll_y),
            None if self.auto_size else (0, 0, self.w, self.h)
        )

    def _on_mouse_wheel(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_wheel(  # type: ignore
                (event.x, event.y), hasattr(event, 'touch') and event.touch, event.flipped
            )

    def _on_mouse_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_leave(event.pos)  # type: ignore

    def _on_mouse_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_enter(event.pos)  # type: ignore

    def _on_mouse_move(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_move(  # type: ignore
                event.pos, event.rel, event.buttons, hasattr(event, 'touch') and event.touch
            )

    def _on_mouse_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_down(event.pos, event.button)  # type: ignore

    def _on_mouse_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_mouse_up(event.pos, event.button)  # type: ignore
        if event.button == pygame.BUTTON_LEFT and hasattr(self, 'on_click'):
            self.on_click(event.pos)

    def _on_focus_enter(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_enter(event.pos, event.button)  # type: ignore

    def _on_focus_leave(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_focus_leave(event.pos, event.button)  # type: ignore

    def _on_key_down(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_down(event)  # type: ignore

    def _on_key_up(self, event: pygame.event.Event, bind: bool) -> None:
        if bind:
            self.on_key_up(event)  # type: ignore
